@@ -19,11 +19,20 @@ AnnotationHub <-
              ask=getAnnotationHubOption("ASK"))
 {
     if (is.null(proxy)){
-        connect <- !is.null(curl::nslookup("annotationhub.bioconductor.org", error=FALSE))
+        connect <- suppressWarnings(tryCatch({
+            readBin(hub, n=1L, what="raw")
+            TRUE
+        }, error = function(...){
+            FALSE
+        }))
     } else {
         connect <- TRUE
-        message("Assuming valid proxy connection through '", proxy, "'",
-                "\n  If you experience connection issues consider ",
+        message("Assuming valid proxy connection through '",
+                ifelse(is(proxy,"request"),
+                       paste(unlist(proxy), collapse=":"),
+                       proxy),
+                "'",
+                "\n If you experience connection issues consider ",
                 "using 'localHub=TRUE'")
     }
     if (!connect && !localHub){
