@@ -20,11 +20,11 @@ setClass("Hub",
 ### Constructor for subclasses
 ###
 
-.Hub <- function(.class, url, cache, proxy, localHub=FALSE, ...) {
+.Hub <- function(.class, url, cache, proxy, localHub=FALSE, ask=TRUE, ...) {
 
     # create or use cache location
     # download hub sqlite file and add/update to BiocFileCache for tracking
-    db_path <- .create_cache(.class, url, cache, proxy, localHub)
+    db_path <- .create_cache(.class, url, cache, proxy, localHub, ask)
 
     # find snapshot date
     if (!localHub){
@@ -288,13 +288,24 @@ setMethod("[[", c("Hub", "character", "missing"),
         stop("'i' must be length 1")
     idx <- match(i, names(.db_uid(x)))
     if (is.na(idx)){
+        status = recordStatus(x, i)
         if (isLocalHub(x)){
-            if(recordStatus(x, i)$status == "Public")
+            if(status$status == "Public")
                 stop("File not previously downloaded.",
                      "\n  Run with 'localHub=FALSE'",
                      call.=FALSE)
         }
-        stop(recordStatus(x, i)$status, call.=FALSE)
+        if (status$status == "Public" && (as.Date(status$dateadded) >=
+                as.Date(snapshotDate(x))))
+            stop(status$record, " added after current Hub snapshot date.\n",
+                 "  added: ", as.character(status$dateadded), "\n",
+                 "  snapshote date: ",  as.character(snapshotDate(x)),
+                 call.=FALSE)
+
+        msg <- paste0(status$status, "\n")
+        if(!is.null(status$dateremoved))
+            msg <- paste0(msg, "   Resource removed on: ", status$dateremoved)
+        stop(msg, call.=FALSE)
     }
     .Hub_get1(x[idx], force=force, verbose=verbose)
 })
@@ -338,9 +349,116 @@ setReplaceMethod("cache", "Hub",
     if (length(rmid) > 0){
         bfcremove(bfc, rids=rmid)
     }
+
+    # reset hub = important especially if using localHub
+    # will not reset original object but will reset subsetting object
+    localHub <- isLocalHub(x)
+    db_path <- x@.db_path
+    if (!localHub){
+        db_date <- .restrictDateByVersion(db_path)
+    } else {
+        dates <-.possibleDates(db_path)
+        db_date <- dates[length(dates)]
+    }
+    db_uid <- .db_uid0(db_path, db_date, localHub)
+    hub <- new(as.character(class(x)), cache=hubCache(x), hub=hubUrl(x),
+               date=db_date, .db_path=db_path, .db_uid=db_uid,
+               isLocalHub=localHub)
+    if (!localHub){
+        index <- .db_create_index(hub)
+        .db_index(hub) <- index
+    } else{
+        index <- .db_index_file(hub)
+        .db_index(hub) <- index
+        hub <- .subsethub(hub)
+    }
+    hub
 })
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Removing Resources from downloaded cache
+###
+
+setMethod("removeResources", "missing",
+    function(hub, ids)
+{
+    ids <- names(.db_uid(hub))
+    message("Removing all resources from cache")
+    suppressMessages(removeResources(hub, ids))
+})
+
+setMethod("removeResources", "character",
+    function(hub, ids)
+{
+    alldatainfo <- .IdsInfo(hub)
+    dx <- match(ids, alldatainfo$ah_id)
+    if(any(is.na(dx))){
+        message("Not all ids found in database.",
+                "\n  Ignoring the following:",
+                "\n      ",
+                paste(ids[is.na(dx)], collapse="\n      "))
+    }
+
+    if(length(dx[!is.na(dx)]) > 0L){
+
+        ids <- ids[!is.na(dx)]
+        bfc <-  BiocFileCache(cache=hubCache(hub))
+        rnames <- bfcinfo(bfc)$rname
+        cachepath <- .named_cache_path(hub)
+        cachepath <- cachepath[(names(cachepath) %in% ids)]
+        Toremove <- paste(names(cachepath), cachepath, sep=" : ")
+        if (length(Toremove) > 0){
+            bfc_ids <- bfcinfo(bfc)[match(Toremove, rnames),]$rid
+            if (any(is.na(bfc_ids))){
+                message("The following are not locally downloaded:",
+                        "\n      ",
+                        paste(names(cachepath)[which(is.na(bfc_ids))],
+                        collapse="\n      "))
+                cachepath <- cachepath[which(!is.na(bfc_ids))]
+                bfc_ids <- bfc_ids[which(!is.na(bfc_ids))]
+            }
+            if (length(bfc_ids) > 0){
+
+                message("Removing the following from cache:",
+                        "\n      ",
+                        paste(names(cachepath), collapse = "\n      "))
+                bfc <- bfcremove(x=bfc, rids=bfc_ids)
+
+                # reset hub cache - useful if local
+                localHub <- isLocalHub(hub)
+                db_path <- hub@.db_path
+                if (!localHub){
+                    db_date <- .restrictDateByVersion(db_path)
+                } else {
+                    dates <-.possibleDates(db_path)
+                    db_date <- dates[length(dates)]
+                }
+                db_uid <- .db_uid0(db_path, db_date, localHub)
+                hub <- new(as.character(class(hub)), cache=hubCache(hub), hub=hubUrl(hub),
+                           date=db_date, .db_path=db_path, .db_uid=db_uid,
+                           isLocalHub=localHub)
+                if (!localHub){
+                    index <- .db_create_index(hub)
+                    .db_index(hub) <- index
+                } else{
+                    index <- .db_index_file(hub)
+                    .db_index(hub) <- index
+                    hub <- .subsethub(hub)
+                }
+            }
+        } else {
+            message("Resources were not locally cached.",
+                    "\n    Nothing to remove.")
+        }
+    } else {
+
+        stop("No valid ids to remove.")
+
+    }
+    invisible(hub)
+})
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### $, query() and subset() methods
 ###
@@ -545,6 +663,12 @@ setMethod("as.list", "Hub", as.list.Hub)
 
 setMethod("show", "Hub", function(object)
 {
+    len <- length(object)
+    cat(
+        class(object), " with ", len, " record", ifelse(len==1L, "", "s"), "\n",
+        "# snapshotDate(): ", snapshotDate(object), "\n",
+        sep = ""
+    )
     if (length(object) == 1)
         .show1(object)
     else
@@ -594,14 +718,23 @@ setMethod("recordStatus", "Hub",
         if (length(record) != 1L)
             stop("'record' must be length 1")
 
-        conn <- dbconn(hub)
-        query <- paste0("SELECT status FROM statuses WHERE id IN ",
-                        "(SELECT status_id FROM resources WHERE ah_id = '",
-                        record, "')")
-        status=.db_query(conn, query)
+        query <- paste0("SELECT DISTINCT resources.ah_id, statuses.status, ",
+                        "resources.rdatadateadded, resources.rdatadateremoved ",
+                        "FROM resources, statuses ",
+                        "WHERE resources.status_id == statuses.id ",
+                        "AND ah_id = '", record, "'")
+
+        status=.db_query(dbfile(hub), query)
         if (nrow(status) == 0L)
-            status <- "record not found in database"
-        data.frame(record=record, status=status)
+            return(data.frame(record=record, status="record not found in database"))
+        if(is.na(status$rdatadateremoved)){
+            return(data.frame(record=record, status=status$status,
+                              dateadded=status$rdatadateadded))
+        }else{
+            return(data.frame(record=record, status=status$status,
+                              dateadded=status$rdatadateadded,
+                              dateremoved=status$rdatadateremoved))
+        }
 })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
