@@ -1,7 +1,7 @@
 .create_cache <-
-    function(.class, url, cache, proxy, localHub=FALSE)
+    function(.class, url, cache, proxy, localHub=FALSE, ask=TRUE)
 {
-    hub_bfc <- BiocFileCache(cache=cache)
+    hub_bfc <- BiocFileCache(cache=cache, ask=ask)
     db_path <- .updateHubDB(hub_bfc, .class, url, proxy, localHub)
     .db_is_valid(db_path)
     db_path
@@ -148,7 +148,15 @@ removeCache <- function(x, ask=TRUE){
         } else if (cnt == 0){
             TRUE
         } else {
-            bfcneedsupdate(bfc, rids=rid)
+            tryCatch({
+                bfcneedsupdate(bfc, rids=rid)
+            }, error=function(e){
+                ahidnf <- res %>% collect(Inf) %>% `[[`("rname") %>%
+                    strsplit(split=" : ") %>% `[[`(1) %>% `[`(1)
+                message("Could not check id: ",ahidnf," for updates.",
+                        "\n  Using previously cached version.")
+                setNames(FALSE, rid)
+            })
         }
     }
     if (length(fndFiles) > 0){
