@@ -1,6 +1,6 @@
 ## This function filters the local annotationhub.sqlite metadata db and
 ## defines the subset exposed by AnnotationHub().
-.uid0 <- function(path, date)
+.uid0 <- function(path, date, localHub)
 {
     conn <- .db_open(path)
     on.exit(.db_close(conn))
@@ -21,16 +21,24 @@
     ##   Bioconductor was valid.
     ##   NOTE: The 'date' variable is the snapshotDate().
 
+    # Ran into an issue comparing BiocVersion once we hit 3.10
+    # 3.10 got truncated to 3.1 and missed values
+    bioc_value <- .db_query(conn,
+                            "SELECT DISTINCT biocversion FROM biocversions")[[1]]
+    indx <- package_version(bioc_value) <= BiocManager::version()
+    if (sum(indx) != 0) bioc_value <- bioc_value[indx]
+    bioc_value <- paste(paste0('"', bioc_value, '"'), collapse=",")
+
     query1 <- sprintf(
         'SELECT resources.id
          FROM resources, rdatapaths, biocversions
          WHERE resources.rdatadateadded <= "%s"
-         AND biocversions.biocversion <= "%s"
+         AND biocversions.biocversion IN (%s)
          AND resources.rdatadateremoved IS NULL
          AND rdatapaths.rdataclass != "OrgDb"
          AND biocversions.resource_id == resources.id
          AND rdatapaths.resource_id == resources.id',
-         date, BiocManager::version())
+         date, bioc_value)
     biocIds1 <- .db_query(conn, query1)[[1]]
 
     ## Add a query to get resources that have been removed
@@ -42,12 +50,12 @@
         'SELECT resources.id
          FROM resources, rdatapaths, biocversions
          WHERE resources.rdatadateadded <= "%s"
-         AND biocversions.biocversion <= "%s"
+         AND biocversions.biocversion IN (%s)
          AND resources.rdatadateremoved > "%s"
          AND rdatapaths.rdataclass != "OrgDb"
          AND biocversions.resource_id == resources.id
          AND rdatapaths.resource_id == resources.id',
-         date, BiocManager::version(), date)
+         date, bioc_value, date)
     biocIds3 <- .db_query(conn, query3)[[1]]
     ## OrgDb sqlite files:
     ##
@@ -67,23 +75,38 @@
     ## NOTE: Because OrgDbs are valid for a full devel cycle they are
     ##       not filtered by snapshotDate(); the OrgDbs are valid for all
 
-    isDevel <- BiocManager:::isDevel()
-    orgdb_release_version <- ifelse(getAnnotationHubOption("TESTING"),
-                                    as.character(BiocManager::version()),
-                                    ifelse(isDevel,
-                                           as.character(BiocManager:::.version_bioc("release")),
-                                           as.character(BiocManager::version())))
+    if(curl::has_internet() || !localHub){
+        isDevel <- BiocManager:::isDevel()
+        orgdb_release_version <-
+            if (getAnnotationHubOption("TESTING") || !isDevel) {
+                BiocManager::version()
+            } else {
+                BiocManager:::.version_bioc("release")
+            }
+
+        query2 <- sprintf(
+            'SELECT resources.id
+            FROM resources, biocversions, rdatapaths
+            WHERE biocversions.biocversion == "%s"
+            AND rdatapaths.rdataclass == "OrgDb"
+            AND resources.rdatadateremoved IS NULL
+            AND biocversions.resource_id == resources.id
+            AND rdatapaths.resource_id == resources.id',
+            orgdb_release_version)
+        biocIds2 <- .db_query(conn, query2)[[1]]
+    }else{
+        query2 <- sprintf(
+            'SELECT resources.id
+            FROM resources, biocversions, rdatapaths
+            WHERE biocversions.biocversion == "%s"
+            AND rdatapaths.rdataclass == "OrgDb"
+            AND resources.rdatadateremoved IS NULL
+            AND biocversions.resource_id == resources.id
+            AND rdatapaths.resource_id == resources.id',
+            BiocManager::version())
+        biocIds2 <- .db_query(conn, query2)[[1]]
+    }
 
-    query2 <- sprintf(
-        'SELECT resources.id
-         FROM resources, biocversions, rdatapaths
-         WHERE biocversions.biocversion == "%s"
-         AND rdatapaths.rdataclass == "OrgDb"
-         AND resources.rdatadateremoved IS NULL
-         AND biocversions.resource_id == resources.id
-         AND rdatapaths.resource_id == resources.id',
-         orgdb_release_version)
-    biocIds2 <- .db_query(conn, query2)[[1]]
 
     ## make unique and sort
     allIds = sort(unique(c(biocIds1, biocIds2, biocIds3)))
